# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.2

import enum

import winrt.system
from winrt import _winrt_windows_devices_wifidirect

__all__ = [
    "WiFiDirectAdvertisementListenStateDiscoverability",
    "WiFiDirectAdvertisementPublisherStatus",
    "WiFiDirectConfigurationMethod",
    "WiFiDirectConnectionStatus",
    "WiFiDirectDeviceSelectorType",
    "WiFiDirectError",
    "WiFiDirectPairingProcedure",
    "WiFiDirectAdvertisement",
    "WiFiDirectAdvertisementPublisher",
    "WiFiDirectAdvertisementPublisherStatusChangedEventArgs",
    "WiFiDirectConnectionListener",
    "WiFiDirectConnectionParameters",
    "WiFiDirectConnectionRequest",
    "WiFiDirectConnectionRequestedEventArgs",
    "WiFiDirectDevice",
    "WiFiDirectInformationElement",
    "WiFiDirectLegacySettings",
]

class WiFiDirectAdvertisementListenStateDiscoverability(enum.IntEnum):
    NONE = 0
    NORMAL = 1
    INTENSIVE = 2

class WiFiDirectAdvertisementPublisherStatus(enum.IntEnum):
    CREATED = 0
    STARTED = 1
    STOPPED = 2
    ABORTED = 3

class WiFiDirectConfigurationMethod(enum.IntEnum):
    PROVIDE_PIN = 0
    DISPLAY_PIN = 1
    PUSH_BUTTON = 2

class WiFiDirectConnectionStatus(enum.IntEnum):
    DISCONNECTED = 0
    CONNECTED = 1

class WiFiDirectDeviceSelectorType(enum.IntEnum):
    DEVICE_INTERFACE = 0
    ASSOCIATION_ENDPOINT = 1

class WiFiDirectError(enum.IntEnum):
    SUCCESS = 0
    RADIO_NOT_AVAILABLE = 1
    RESOURCE_IN_USE = 2

class WiFiDirectPairingProcedure(enum.IntEnum):
    GROUP_OWNER_NEGOTIATION = 0
    INVITATION = 1

WiFiDirectAdvertisement = _winrt_windows_devices_wifidirect.WiFiDirectAdvertisement
WiFiDirectAdvertisementPublisher = _winrt_windows_devices_wifidirect.WiFiDirectAdvertisementPublisher
WiFiDirectAdvertisementPublisherStatusChangedEventArgs = _winrt_windows_devices_wifidirect.WiFiDirectAdvertisementPublisherStatusChangedEventArgs
WiFiDirectConnectionListener = _winrt_windows_devices_wifidirect.WiFiDirectConnectionListener
WiFiDirectConnectionParameters = _winrt_windows_devices_wifidirect.WiFiDirectConnectionParameters
WiFiDirectConnectionRequest = _winrt_windows_devices_wifidirect.WiFiDirectConnectionRequest
WiFiDirectConnectionRequestedEventArgs = _winrt_windows_devices_wifidirect.WiFiDirectConnectionRequestedEventArgs
WiFiDirectDevice = _winrt_windows_devices_wifidirect.WiFiDirectDevice
WiFiDirectInformationElement = _winrt_windows_devices_wifidirect.WiFiDirectInformationElement
WiFiDirectLegacySettings = _winrt_windows_devices_wifidirect.WiFiDirectLegacySettings
